﻿using System.Collections.Generic;
using System.Drawing;
using System;

namespace LegoPixelSpaceInvaders
{
    public class Swarm
    {
        private List<Alien> m_Aliens;
        private Projectile m_Projectile;
        private Missile m_Missile;
        private Point m_Position;
        private bool m_Dir;
        private bool m_Apocalypse;
        private Color m_Background;
        private static Random m_Rnd = new Random();
        private int m_ApoStep;
        private int m_PacDeath;

        public Swarm(Point position)
        {
            m_Aliens = new List<Alien>();
            m_Position = position;
            m_Projectile = new Projectile();
            m_Missile = new Missile();
            m_Background = Color.Black;

            for (int y = 0; y < 3; y++)
            {
                for (int x = 0; x < 6; x++)    
                    m_Aliens.Add(new Alien(y % 2 == 0, new Point(m_Position.X + x * 15, m_Position.Y + y * 11)));
            }
        }

        public int PacDeath { get { return m_PacDeath; } }
        public Point PacPos { get { return m_Aliens[7].Position; } }

        public void Shoot(Point p)
        {
            m_Projectile.Position = p;
        }

        public void ShootMissile(Point p)
        {
            if(!m_Apocalypse)
                m_Missile.Position = p;
        }

        public void Update(IAudioContext context)
        {
            if (m_Apocalypse)
            {
                for(int i = 0; i < m_Aliens.Count; i++)
                {
                    if (i != 7)
                    {
                        Alien alien = m_Aliens[i];
                        alien.Update(context);

                        int offx = m_Rnd.Next(-1, 2);
                        alien.Position = new Point(alien.Position.X + offx, alien.Position.Y + 1);

                        int color = alien.Color.R;
                        color += m_ApoStep;

                        if (color < 128 && color >= 0)
                            alien.Color = Color.FromArgb(color, color, color);
                        else
                            m_ApoStep = -15;

                        if (color < 0)
                        {
                            context.PlaySound("godlike", 0);
                            m_ApoStep = 0;
                        }

                    }
                }

                m_Aliens[7].Update(context);

                m_Projectile.Update(context);

                for (int i = m_Aliens.Count - 1; i >= 0; i--)
                {
                    if (m_Aliens[i].Position.Y > -8)
                    {
                        if (m_Aliens[i].IsInside(m_Projectile))
                        {
                            m_Projectile.Idle = true;
                            m_PacDeath++;
                        }
                    }
                }

                int bgc = m_Background.R;
                bgc -= 15;

                if(bgc >= 0)
                    m_Background = Color.FromArgb(bgc, bgc, bgc);
            }
            else
            {
                if (m_Dir)
                    m_Position.X++;
                else
                    m_Position.X--;

                if (m_Position.X + 86 > 111)
                {
                    m_Position.Y++;
                    m_Position.X--;
                    m_Dir = !m_Dir;
                }

                if (m_Position.X < 2)
                {
                    m_Position.Y++;
                    m_Position.X++;
                    m_Dir = !m_Dir;
                }

                m_Aliens.ForEach(alien => alien.Update(context));

                for (int x = 0; x < 6; x++)
                {
                    for (int y = 0; y < 3; y++)
                        m_Aliens[y * 6 + x].Position = new Point(m_Position.X + x * 15, m_Position.Y + y * 11);
                }

                m_Projectile.Update(context);
                m_Missile.Update(context);

                for (int i = m_Aliens.Count - 1; i >= 0; i--)
                {
                    if (m_Aliens[i].Position.Y > -8)
                    {
                        if (m_Aliens[i].IsInside(m_Projectile))
                        {
                            m_Projectile.Idle = true;
                            m_Aliens[i].Idle = true;
                        }

                        if (m_Aliens[i].IsInside(m_Missile))
                        {
                            m_Missile.Idle = true;
                            context.PlaySound("nuklearschlag", 0);
                            startApocalypse();
                        }
                    }
                }
            }
        }

        public void startApocalypse()
        {
            m_Aliens[7].NuclearSurvive();

            for (int i = 0; i < m_Aliens.Count; i++)
            {
                if(i != 7)
                    m_Aliens[i].NuclearKill();
            }

            m_Background = Color.White;
            m_ApoStep = 10;
            m_Apocalypse = true;
        }

        public void Draw(IDrawingContext context)
        {
            for(int y = 0; y < 64; y++)
            {
                for (int x = 0; x < 113; x++)
                    context.DrawPixel(x, y, m_Background);
            }

            m_Aliens.ForEach(alien => alien.Draw(context));
            m_Projectile.Draw(context);
            m_Missile.Draw(context);
        }
    }
}
